import numpy as np
import scipy.optimize
import scipy.stats

from artist import Plot


def main():
    # Draw random numbers from the normal distribution
    np.random.seed(1)
    N = np.random.normal(size=2000)

    # define bin edges
    edge = 5
    bin_width = .1
    bins = np.arange(-edge, edge + .5 * bin_width, bin_width)

    # build histogram and x, y values at the center of the bins
    n, bins = np.histogram(N, bins=bins)
    x = (bins[:-1] + bins[1:]) / 2
    y = n

    # fit normal distribution pdf to data
    f = lambda x, N, mu, sigma: N * scipy.stats.norm.pdf(x, mu, sigma)
    popt, pcov = scipy.optimize.curve_fit(f, x, y)
    print("Parameters from fit (N, mu, sigma):", popt)

    # make graph
    graph = Plot()

    # graph histogram
    graph.histogram(n, bins)

    # graph model with fit parameters
    x = np.linspace(-edge, edge, 100)
    graph.plot(x, f(x, *popt), mark=None)

    # set labels and limits
    graph.set_xlabel("value")
    graph.set_ylabel("count")
    graph.set_label("Fit to data")
    graph.set_xlimits(-6, 6)

    # save graph to file
    graph.save('histogram-fit')


if __name__ == '__main__':
    main()



"""Create a plot.

Contents
--------

:class:`BasePlotContainer`
    Base class for standalone plots.

:class:`SubPlot`
    Plot data in a data rectangle.

:class:`Plot`
    Create a plot containing a single subplot.

:class:`PolarPlot`
    Create a plot containing a single polar subplot.

"""

import subprocess
import os
import tempfile
import shutil
try:
    # Python 2
    from itertools import izip_longest
except ImportError:
    # Python 3
    from itertools import zip_longest as izip_longest

from math import log10, sqrt

from PIL import Image
import jinja2
import numpy as np


RELATIVE_NODE_LOCATIONS = {'upper right': {'node_location': 'below left',
                                           'x': 1, 'y': 1},
                           'upper left': {'node_location': 'below right',
                                          'x': 0, 'y': 1},
                           'lower left': {'node_location': 'above right',
                                          'x': 0, 'y': 0},
                           'lower right': {'node_location': 'above left',
                                           'x': 1, 'y': 0},
                           'center': {'node_location': 'center',
                                      'x': 0.5, 'y': 0.5}}


class BasePlotContainer(object):

    """Base class for stand-alone plots.

    This class provides methods for rendering the plot.  To provide
    methods for plotting and annotating, subclass this base class.

    """

    # The templates must be initialized on instantiation.
    template = None
    document_template = None

    def render(self, template=None):
        """Render the plot using a template.

        Once the plot is complete, it needs to be rendered.  Artist uses
        the Jinja2 templating engine.  The default template results in a
        LaTeX file which can be included in your document.

        :param template: a user-supplied template or None.
        :type template: string or None.
        :returns: the rendered template as string.

        This is a very minimal implementation.  Override this method to
        include variables in the template.render call.

        """
        if not template:
            template = self.template

        response = template.render()
        return response

    def save_assets(self, dest_path):
        """Save plot assets alongside dest_path.

        Some plots may have assets, like bitmap files, which need to be
        saved alongside the rendered plot file.

        :param dest_path: path of the main output file.

        """
        pass

    def render_as_document(self):
        """Render the plot as a stand-alone document.

        :returns: the rendered template as string.

        """
        return self.render(self.document_template)

    def save(self, dest_path):
        r"""Save the plot as a includable LaTeX file.

        The output file can be included (using \input) in your LaTeX
        document.

        :param dest_path: path of the file.

        """
        self.save_assets(dest_path)
        self.external_filename = 'externalized-' + os.path.basename(dest_path)
        dest_path = self._add_extension('tex', dest_path)
        with open(dest_path, 'w') as f:
            f.write(self.render())

    def save_as_document(self, dest_path):
        """Save the plot as a stand-alone LaTeX file.

        :param dest_path: path of the file.

        """
        self.save_assets(dest_path)
        self.external_filename = 'externalized-' + os.path.basename(dest_path)
        dest_path = self._add_extension('tex', dest_path)
        with open(dest_path, 'w') as f:
            f.write(self.render_as_document())

    def save_as_pdf(self, dest_path):
        """Save the plot as a PDF file.

        Save and render the plot using LaTeX to create a PDF file.

        :param dest_path: path of the file.

        """
        dest_path = self._add_extension('pdf', dest_path)
        build_dir = tempfile.mkdtemp()
        build_path = os.path.join(build_dir, 'document.tex')
        self.save_assets(build_path)
        with open(build_path, 'w') as f:
            f.write(self.render_as_document())
        pdf_path = self._build_document(build_path)
        self._crop_document(pdf_path)
        shutil.copyfile(pdf_path, dest_path)
        shutil.rmtree(build_dir)

    def _build_document(self, path):
        dir_path = os.path.dirname(path)

        cwd = os.getcwd()
        os.chdir(dir_path)

        try:
            subprocess.check_output(['pdflatex', '-halt-on-error', path],
                                    stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as exc:
            output_lines = exc.output.split('\n')
            error_lines = [line for line in output_lines
                           if line and line[0] == '!']
            errors = '\n'.join(error_lines)
            raise RuntimeError("LaTeX compilation failed:\n" + errors +
                               "\nTemp build dir: " + dir_path)
        finally:
            os.chdir(cwd)

        pdf_path = path.replace('.tex', '.pdf')
        return pdf_path

    def _crop_document(self, path):
        dirname = os.path.dirname(path)
        output_path = os.path.join(dirname, 'crop-output.pdf')
        try:
            subprocess.check_output(['pdfcrop', path, output_path],
                                    stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as exc:
            raise RuntimeError("Cropping PDF failed:\n" + exc.output)
        os.rename(output_path, path)

    def _add_extension(self, extension, path):
        if '.' not in path:
            return path + '.' + extension
        else:
            return path

    def _convert_none(self, variable):
        if variable is not None:
            return variable
        else:
            return ''

    def _get_axis_modes(self, axis):
        if axis == 'loglog':
            return 'log', 'log'
        elif axis == 'semilogx':
            return 'log', 'normal'
        elif axis == 'semilogy':
            return 'normal', 'log'
        else:
            return 'normal', 'normal'


class SubPlot(object):

    """Plot data in a data rectangle.

    Provides methods to plot or histogram data, shade regions, add labels,
    pins and titles.  This class is not meant to be used directly.
    Instead, use the Plot class for single plots, or the MultiPlot class
    for plots containing multiple subplots.

    """

    def __init__(self):
        self.shaded_regions_list = []
        self.plot_series_list = []
        self.plot_table_list = []
        self.histogram2d_list = []
        self.bitmap_list = []
        self.pin_list = []
        self.horizontal_lines = []
        self.vertical_lines = []
        self.axis_background = None
        self.title = None
        self.xlabel = None
        self.ylabel = None
        self.label = None
        self.limits = {'xmin': None, 'xmax': None,
                       'ymin': None, 'ymax': None,
                       'mmin': None, 'mmax': None,
                       'smin': None, 'smax': None}
        self.ticks = {'x': [], 'y': [],
                      'xlabels': '', 'ylabels': '',
                      'xsuffix': '', 'ysuffix': ''}
        self.axis_equal = False
        self.scalebar = None
        self.colorbar = False
        self.colormap = None

    def save_assets(self, dest_path, suffix=''):
        """Save plot assets alongside dest_path.

        Some plots may have assets, like bitmap files, which need to be
        saved alongside the rendered plot file.

        :param dest_path: path of the main output file.
        :param suffix: optional suffix to add to asset names.

        """
        self._write_bitmaps(dest_path, suffix)

    def plot(self, x, y, xerr=[], yerr=[], mark='o',
             linestyle='solid', use_steps=False, markstyle=None):
        """Add a data series to the plot.

        :param x: array containing x-values.
        :param y: array containing y-values.
        :param xerr: (optional) array containing errors on the x-values.
        :param yerr: (optional) array containing errors on the y-values.
        :param mark: the symbol used to mark the data point.  May be None,
            or any plot mark accepted by TikZ (e.g. *, x, +, o, square,
            triangle).
        :param linestyle: the line style used to connect the data points.
            May be None, or any line style accepted by TikZ (e.g. solid,
            dashed, dotted, thick, or even combinations like
            "red,thick,dashed").
        :param use_steps: if True, draw a stepped plot.
        :param markstyle: the style of the plot marks (e.g. 'mark
            size=.75pt')

        The dimensions of x, y, xerr and yerr should be equal.  However,
        xerr and yerr may be empty lists.

        """
        # clear the background of the marks
        self._clear_plot_mark_background(x, y, mark, markstyle)
        # draw the plot series over the background
        options = self._parse_plot_options(mark, linestyle, use_steps,
                                           markstyle)
        plot_series = self._create_plot_series_object(x, y, xerr, yerr,
                                                      options)
        self.plot_series_list.append(plot_series)

    def _clear_plot_mark_background(self, x, y, mark, markstyle):
        options = self._create_mark_background_options(mark, markstyle)
        if options:
            plot_series = self._create_plot_series_object(x, y,
                                                          options=options)
            # make sure all background clear operations are performed first
            self.plot_series_list.insert(0, plot_series)

    def _create_plot_series_object(self, x, y, xerr=[], yerr=[], options=None):
        return {'options': options,
                'data': list(izip_longest(x, y, xerr, yerr)),
                'show_xerr': True if len(xerr) else False,
                'show_yerr': True if len(yerr) else False}

    def histogram(self, counts, bin_edges, linestyle='solid'):
        """Plot a histogram.

        The user needs to supply the histogram.  This method only plots
        the results.  You can use NumPy's histogram function.

        :param counts: array containing the count values.
        :param bin_edges: array containing the bin edges.
        :param linestyle: the line style used to connect the data points.
            May be None, or any line style accepted by TikZ (e.g. solid,
            dashed, dotted, thick, or even combinations like
            "red,thick,dashed").

        Example::

            >>> plot = artist.Plot()
            >>> x = np.random.normal(size=1000)
            >>> n, bins = np.histogram(x)
            >>> plot.histogram(n, bins)

        """
        if len(bin_edges) - 1 != len(counts):
            raise RuntimeError(
                "The length of bin_edges should be length of counts + 1")
        x = bin_edges
        y = list(counts) + [counts[-1]]
        self.plot(x, y, mark=None, linestyle=linestyle, use_steps=True)

    def histogram2d(self, counts, x_edges, y_edges, type='bw', style=None,
                    bitmap=False):
        """Plot a two-dimensional histogram.

        The user needs to supply the histogram.  This method only plots
        the results.  You can use NumPy's histogram2d function.

        :param counts: array containing the count values.
        :param x_edges: array containing the x-axis bin edges.
        :param y_edges: array containing the y-axis bin edges.
        :param type: the type of histogram.  Allowed values are 'bw' for
            filled squares with shades from black (minimum value) to white
            (maximum value), 'reverse_bw' for filled squares with the
            shades reversed and 'area' for squares where the area of the
            square is a measure of the count in the bin.
        :param style: optional TikZ styles to apply (e.g. 'red').  Note
            that many color styles are overridden by the 'bw' and
            'reverse_bw' types.
        :param bitmap: Export the histogram as an image for better
            performance. This does expect all bins along an axis to have
            equal width. Can only be combined with type 'bw' and
            'reverse_bw'.

        """
        if counts.shape != (len(x_edges) - 1, len(y_edges) - 1):
            raise RuntimeError(
                "The length of x_edges and y_edges should match counts")

        if type not in ['bw', 'reverse_bw', 'area']:
            raise RuntimeError("Histogram type %s not supported" % type)
        if type == 'area' and bitmap:
            raise RuntimeError("Histogram type %s not supported for bitmap "
                               "output" % type)

        if bitmap:
            normed_counts = self._normalize_histogram2d(counts, type)
            img = Image.fromarray(np.flipud(normed_counts.T))
            self.bitmap_list.append({'image': img,
                                     'xmin': min(x_edges),
                                     'xmax': max(x_edges),
                                     'ymin': min(y_edges),
                                     'ymax': max(y_edges)})
        else:
            x_centers = (x_edges[:-1] + x_edges[1:]) / 2
            y_centers = (y_edges[:-1] + y_edges[1:]) / 2

            self.histogram2d_list.append({'x_edges': x_edges,
                                          'y_edges': y_edges,
                                          'x_centers': x_centers,
                                          'y_centers': y_centers,
                                          'counts': counts,
                                          'max': counts.max(),
                                          'type': type,
                                          'style': style})
        # Set limits unless lower/higher limits are already set.
        xmin = min(x for x in (min(x_edges), self.limits['xmin'])
                   if x is not None)
        ymin = min(y for y in (min(y_edges), self.limits['ymin'])
                   if y is not None)
        xmax = max(x for x in (max(x_edges), self.limits['xmax'])
                   if x is not None)
        ymax = max(y for y in (max(y_edges), self.limits['ymax'])
                   if y is not None)
        self.set_xlimits(xmin, xmax)
        self.set_ylimits(ymin, ymax)

    def scatter(self, x, y, mark='o', markstyle=None):
        """Plot a series of points.

        Plot a series of points (marks) that are not connected by a
        line. Shortcut for plot with linestyle=None.

        :param x: array containing x-values.
        :param y: array containing y-values.
        :param mark: the symbol used to mark the data points. May be
            any plot mark accepted by TikZ (e.g. *, x, +, o, square,
            triangle).
        :param markstyle: the style of the plot marks (e.g. 'mark
            size=.75pt')

        Example::

            >>> plot = artist.Plot()
            >>> x = np.random.normal(size=20)
            >>> y = np.random.normal(size=20)
            >>> plot.scatter(x, y, mark='*')

        """
        self.plot(x, y, mark=mark, linestyle=None, markstyle=markstyle)

    def scatter_table(self, x, y, c, s, mark='*'):
        """Add a data series to the plot.

        :param x: array containing x-values.
        :param y: array containing y-values.
        :param c: array containing values for the color of the mark.
        :param s: array containing values for the size of the mark.
        :param mark: the symbol used to mark the data point.  May be None,
            or any plot mark accepted by TikZ (e.g. *, x, +, o, square,
            triangle).

        The dimensions of x, y, c and s should be equal. The c values will
        be mapped to a colormap.

        """
        # clear the background of the marks
        # self._clear_plot_mark_background(x, y, mark, markstyle)
        # draw the plot series over the background
        options = self._parse_plot_options(mark)
        s = [sqrt(si) for si in s]
        plot_series = self._create_plot_tables_object(x, y, c, s, options)
        self.plot_table_list.append(plot_series)

    def _create_plot_tables_object(self, x, y, c, s, options=None):
        return {'options': options,
                'data': list(izip_longest(x, y, c, s)),
                'smin': min(s),
                'smax': max(s)}

    def set_title(self, text):
        """Set a title text."""

        self.title = text

    def set_label(self, text, location='upper right', style=None):
        """Set a label for the plot.

        :param text: the label text.
        :param location: the location of the label inside the plot.  May
            be one of 'center', 'upper right', 'lower right', 'upper
            left', 'lower left'.
        :param style: any TikZ style to style the text.

        """
        if location in RELATIVE_NODE_LOCATIONS:
            label = RELATIVE_NODE_LOCATIONS[location].copy()
            label['text'] = text
            label['style'] = style
            self.label = label
        else:
            raise RuntimeError("Unknown label location: %s" % location)

    def add_pin(self, text, location='left', x=None, use_arrow=False,
                relative_position=None, style=None):
        """Add pin to most recent data series.

        :param text: the text of the pin label.
        :param location: the location of the pin relative to the data
            point.  Any location accepted by TikZ is allowed.
        :type location: string
        :param x: the x location of the data point (in the most recent
            data series) at which to place the label.  This is
            interpolated between the actual data points.  If None, only
            the relative_position parameter is used.
        :param use_arrow: specifies whether to draw an arrow between the
            data point and the pin label text.
        :type use_arrow: boolean
        :param relative_position: location of the data point as a relative
            number between 0 and 1.
        :param style: optional TikZ styles to apply (e.g. 'red').

        """
        try:
            series = self.plot_series_list[-1]
        except IndexError:
            raise RuntimeError(
                "First plot a data series, before using this function")

        data = series['data']
        series_x, series_y = list(zip(*data))[:2]

        if x is not None:
            y = np.interp(x, series_x, series_y)
        else:
            x, y = series_x, series_y

        self.add_pin_at_xy(x, y, text, location, relative_position,
                           use_arrow, style)

    def add_pin_at_xy(self, x, y, text, location='above right',
                      relative_position=.9, use_arrow=True, style=None):
        """Add pin at x, y location.

        :param x: array, list or float, specifying the location of the
            pin.
        :param y: array, list or float, specifying the location of the
            pin.
        :param text: the text of the pin label.
        :param location: the location of the pin relative to the data
            point.  Any location accepted by TikZ is allowed.
        :param relative_position: location of the data point as a relative
            number between 0 and 1.
        :param use_arrow: specifies whether to draw an arrow between the
            data point and the pin label text.
        :type use_arrow: boolean
        :param style: optional TikZ styles to apply (e.g. 'red').

        If x, y are arrays or lists, relative position is used to pick a
        point from the arrays.  A relative position of 0.0 will be the
        first point from the series, while 1.0 will be the last point.

        """
        if relative_position is None:
            if location == 'left':
                relative_position = 0.
            elif location == 'right':
                relative_position = 1.
            else:
                relative_position = .8

        x, y = self._calc_position_for_pin(x, y, relative_position)
        self.pin_list.append({'x': x, 'y': y, 'text': text,
                              'location': location,
                              'use_arrow': use_arrow,
                              'options': style})

    def shade_region(self, x, lower, upper, color='lightgray'):
        """Shade a region between upper and lower bounds.

        :param x: array containing x-values
        :param lower: array containing y-values of lower bounds
        :param upper: array containing y-values of upper bounds
        :param color: TikZ style to color the region

        """
        x = list(x)
        reversed_x = list(x)
        reversed_x.reverse()

        lower = list(lower)
        upper = list(upper)
        upper.reverse()

        x = x + reversed_x
        y = lower + upper
        self.shaded_regions_list.append({'data': zip(x, y), 'color': color})

    def draw_image(self, image, xmin=0, ymin=0, xmax=None, ymax=None):
        """Draw an image.

        Do not forget to use :meth:`set_axis_equal` to preserve the
        aspect ratio of the image, or change the aspect ratio of the
        plot to the aspect ratio of the image.

        :param image: Pillow Image object.
        :param xmin,ymin,xmax,ymax: the x, y image bounds.

        Example::

            >>> from PIL import Image
            >>> image = Image.open('background.png')
            >>> height_ratio = (.67 * image.size[1]) / image.size[0]
            >>> plot = artist.Plot(height=r'%.2f\linewidth' % height_ratio)
            >>> plot.draw_image(image)

        """
        if xmax is None:
            xmax = xmin + image.size[0]
        if ymax is None:
            ymax = ymin + image.size[1]
        self.bitmap_list.append({'image': image,
                                 'xmin': xmin,
                                 'xmax': xmax,
                                 'ymin': ymin,
                                 'ymax': ymax})
        # Set limits unless lower/higher limits are already set.
        xmin = min(x for x in (xmin, self.limits['xmin'])
                   if x is not None)
        ymin = min(y for y in (ymin, self.limits['ymin'])
                   if y is not None)
        xmax = max(x for x in (xmax, self.limits['xmax'])
                   if x is not None)
        ymax = max(y for y in (ymax, self.limits['ymax'])
                   if y is not None)
        self.set_xlimits(xmin, xmax)
        self.set_ylimits(ymin, ymax)

    def draw_horizontal_line(self, yvalue, linestyle=None):
        """Draw a horizontal line.

        :param yvalue: y-value of the line
        :param linestyle: TikZ linestyle (e.g. dashed, solid, red)

        """
        self.horizontal_lines.append({'value': yvalue,
                                      'options': linestyle})

    def draw_vertical_line(self, xvalue, linestyle=None):
        """Draw a vertical line.

        :param xvalue: x-value of the line
        :param linestyle: TikZ linestyle (e.g. dashed, solid, red)

        """
        self.vertical_lines.append({'value': xvalue,
                                    'options': linestyle})

    def set_axis_background(self, color='white'):
        """Set a fill color for the axis background.

        :param color: TikZ style to color the axis background.

        """
        self.axis_background = 'fill=%s' % color

    def set_xlabel(self, text):
        """Set a label for the x-axis.

        :param text: text of the label.

        """
        self.xlabel = text

    def set_ylabel(self, text):
        """Set a label for the y-axis.

        :param text: text of the label.

        """
        self.ylabel = text

    def set_xlimits(self, min=None, max=None):
        """Set limits for the x-axis.

        :param min: minimum value to be displayed.  If None, it will be
            calculated.
        :param max: maximum value to be displayed.  If None, it will be
            calculated.

        """
        self.limits['xmin'] = min
        self.limits['xmax'] = max

    def set_ylimits(self, min=None, max=None):
        """Set limits for the y-axis.

        :param min: minimum value to be displayed.  If None, it will be
            calculated.
        :param max: maximum value to be displayed.  If None, it will be
            calculated.

        """
        self.limits['ymin'] = min
        self.limits['ymax'] = max

    def set_mlimits(self, min=None, max=None):
        """Set limits for the point meta (colormap).

        Point meta values outside this range will be clipped.

        :param min: value corresponding to the start of the colormap.
            If None, it will be calculated.
        :param max: value corresponding to the end of the colormap.
            If None, it will be calculated.

        """
        self.limits['mmin'] = min
        self.limits['mmax'] = max

    def set_slimits(self, min, max):
        """Set limits for the size of points in :meth:`scatter_table`.

        If both are None, the size will be the given values.

        :param min: point size for the lowest value.
        :param max: point size for the highest value.

        """
        self.limits['smin'] = sqrt(min)
        self.limits['smax'] = sqrt(max)

    def set_xticks(self, ticks):
        """Set ticks for the x-axis.

        :param ticks: locations for the ticks along the axis.

        """
        self.ticks['x'] = ticks

    def set_yticks(self, ticks):
        """Set ticks for the y-axis.

        :param ticks: locations for the ticks along the axis.

        """
        self.ticks['y'] = ticks

    def set_logxticks(self, logticks):
        """Set ticks for the logarithmic x-axis.

        :param logticks: logarithm of the locations for the ticks along
            the axis.

        For example, if you specify [1, 2, 3], ticks will be placed at 10,
        100 and 1000.

        """
        self.ticks['x'] = ['1e%d' % u for u in logticks]

    def set_logyticks(self, logticks):
        """Set ticks for the logarithmic y-axis.

        :param logticks: logarithm of the locations for the ticks along
            the axis.

        For example, if you specify [1, 2, 3], ticks will be placed at 10,
        100 and 1000.

        """
        self.ticks['y'] = ['1e%d' % u for u in logticks]

    def set_xtick_labels(self, labels):
        """Set tick labels for the x-axis.

        Also set the x-ticks positions to ensure the labels end up on
        the correct place.

        :param labels: list of labels for the ticks along the axis.

        """
        self.ticks['xlabels'] = labels

    def set_ytick_labels(self, labels):
        """Set tick labels for the y-axis.

        Also set the y-ticks positions to ensure the labels end up on
        the correct place.

        :param labels: list of labels for the ticks along the axis.

        """
        self.ticks['ylabels'] = labels

    def set_xtick_suffix(self, suffix):
        """Set the suffix for the ticks of the x-axis.

        :param suffix: string added after each tick. If the value is
                       `degree` or `precent` the corresponding symbols
                       will be added.

        """
        if suffix == 'degree':
            suffix = '^\circ'
        elif suffix == 'percent':
            suffix = '\%'

        self.ticks['xsuffix'] = suffix

    def set_ytick_suffix(self, suffix):
        """Set ticks for the y-axis.

        :param suffix: string added after each tick. If the value is
                       `degree` or `precent` the corresponding symbols
                       will be added.

        """
        if suffix == 'degree':
            suffix = '^\circ'
        elif suffix == 'percent':
            suffix = '\%'

        self.ticks['ysuffix'] = suffix

    def set_axis_equal(self):
        """Scale the axes so the unit vectors have equal length."""

        self.axis_equal = True

    def set_scalebar(self, location='lower right'):
        """Show marker area scale

        :param location: the location of the label inside the plot.  May
            be one of 'center', 'upper right', 'lower right', 'upper
            left', 'lower left'.

        """
        if location in RELATIVE_NODE_LOCATIONS:
            scalebar = RELATIVE_NODE_LOCATIONS[location].copy()
            self.scalebar = scalebar
        else:
            raise RuntimeError("Unknown scalebar location: %s" % location)

    def set_colorbar(self, label='', horizontal=False):
        """Show the colorbar.

        Not for the histogram2d, only for the scatter_table.

        :param label: axis label for the colorbar.
        :param horizontal: boolean, if True the colobar will be horizontal.

        """
        self.colorbar = {'label': label,
                         'horizontal': horizontal}

    def set_colormap(self, name):
        """Choose a colormap for :meth:`scatter_table`.

        :param name: name of the colormap to use. (e.g. hot, cool, blackwhite,
                     greenyellow). If None a coolwarm colormap is used.

        """
        self.colormap = name

    def _parse_plot_options(self, mark=None, linestyle=None,
                            use_steps=False, markstyle=None):
        options = []
        if mark is not None:
            options.append('mark=%s' % mark)
        else:
            options.append('no markers')

        if linestyle is not None:
            options.append(linestyle)
        else:
            options.append('only marks')

        if use_steps is True:
            options.append('const plot')

        if markstyle is not None:
            options.append('mark options={%s}' % markstyle)

        options_string = ','.join(options)
        return options_string

    def _create_mark_background_options(self, mark=None, markstyle=None):
        options = []
        if mark is not None:
            if mark in ['o', 'square', 'triangle', 'diamond', 'pentagon']:
                if mark == 'o':
                    mark = ''
                options.append('mark=%s*,mark options=white,only marks' % mark)
                if markstyle:
                    options.append(',%s' % markstyle)
        options_string = ','.join(options)
        return options_string

    def _calc_position_for_pin(self, x, y, relative_position):
        try:
            max_idx_x = len(x) - 1
            max_idx_y = len(y) - 1
        except TypeError:
            return x, y
        else:
            assert max_idx_x == max_idx_y, \
                'If x and y are iterables, they must be the same length'

            x0, x1 = x[0], x[-1]
            if self.xmode == 'log':
                xs = 10 ** (relative_position * (log10(x1) - log10(x0)) +
                            log10(x0))
            else:
                xs = relative_position * (x1 - x0) + x0
            ys = np.interp(xs, x, y)
            return xs, ys

    def _normalize_histogram2d(self, counts, type):
        """Normalize the values of the counts for a 2D histogram

        This normalizes the values of a numpy array to the range 0-255.

        :param counts: a NumPy array which is to be rescaled.
        :param type: either 'bw' or 'reverse_bw'.

        """
        counts = 255 * (counts - counts.min()) / (counts.max() - counts.min())

        if type == 'reverse_bw':
            counts = 255 - counts

        return counts.astype(np.uint8)

    def _write_bitmaps(self, path, suffix=''):
        """Write bitmap file assets.

        :param path: path of the plot file.
        :param suffix: optional suffix to add to asset names.

        The path parameter is used for the dirname, and the filename.
        So if :meth:`save` is called with '/foo/myplot.tex', you can call
        this method with that same path. The assets will then be saved in
        the /foo directory, and have a name like 'myplot_0.png'.

        """
        dir, prefix = os.path.split(path)
        if '.' in prefix:
            prefix = prefix.split('.')[0]
        if prefix == '':
            prefix = 'figure'
        for i, bitmap in enumerate(self.bitmap_list):
            name = '%s%s_%d.png' % (prefix, suffix, i)
            bitmap['name'] = name
            img = bitmap['image']
            # Make the bitmap at least 1000x1000 pixels
            size0 = int(np.ceil(1000. / img.size[0]) * img.size[0])
            size1 = int(np.ceil(1000. / img.size[1]) * img.size[1])
            large_img = img.resize((size0, size1))
            large_img.save(os.path.join(dir, name))


class Plot(SubPlot, BasePlotContainer):

    """Create a plot containing a single subplot.

    This class creates a 2D plot.  Its various methods add data,
    annotations and options which is stored in class variables.  Finally,
    the plot can be rendered using the Jinja2 templating engine resulting
    in a LaTeX or PDF file.

    """

    def __init__(self, axis='', width=r'.67\linewidth', height=None):
        environment = jinja2.Environment(loader=jinja2.PackageLoader(
            'artist', 'templates'), finalize=self._convert_none)
        self.template = environment.get_template('plot.tex')
        self.document_template = environment.get_template('document.tex')

        self.width = width
        self.height = height
        self.xmode, self.ymode = self._get_axis_modes(axis)
        self.external_filename = None

        super(Plot, self).__init__()

    def render(self, template=None):
        """Render the plot using a template.

        Once the plot is complete, it needs to be rendered.  Artist uses
        the Jinja2 templating engine.  The default template results in a
        LaTeX file which can be included in your document.

        :param template: a user-supplied template or None.
        :type template: string or None.
        :returns: the rendered template as string.

        """
        if not template:
            template = self.template

        response = template.render(
            axis_background=self.axis_background,
            xmode=self.xmode,
            ymode=self.ymode,
            title=self.title,
            width=self.width,
            height=self.height,
            xlabel=self.xlabel,
            ylabel=self.ylabel,
            limits=self.limits,
            ticks=self.ticks,
            axis_equal=self.axis_equal,
            scalebar=self.scalebar,
            colorbar=self.colorbar,
            colormap=self.colormap,
            external_filename=self.external_filename,
            plot=self,
            plot_template=self.template)
        return response


class PolarPlot(Plot):

    """Create a plot containing a single polar subplot.

    Same as the Plot but uses polar axes. The x values are the phi
    coordinates in degrees (or radians). The y values are the r
    coordinates in arbitrary units.

    :param use_radians: If this keyword is set to True the units for x
        values and x axis labels are radians.

    """

    def __init__(self, *args, **kwargs):
        self.use_radians = kwargs.pop('use_radians', False)
        super(PolarPlot, self).__init__(*args, **kwargs)
        environment = jinja2.Environment(loader=jinja2.PackageLoader(
            'artist', 'templates'), finalize=self._convert_none)
        self.template = environment.get_template('polar_plot.tex')

        if not self.use_radians:
            self.set_xtick_suffix('degree')
        else:
            self.set_xticks([0, 30, 60, 90, 120, 150, 180, 210, 240, 270, 300,
                             330])
            self.set_xtick_labels([r'$0$', r'$\frac{1}{6}\pi$',
                                   r'$\frac{2}{6}\pi$', r'$\frac{1}{2}\pi$',
                                   r'$\frac{4}{6}\pi$', r'$\frac{5}{6}\pi$',
                                   r'$\pm\pi$', r'$-\frac{5}{6}\pi$',
                                   r'$-\frac{4}{6}\pi$', r'$-\frac{1}{2}\pi$',
                                   r'$-\frac{2}{6}\pi$', r'$-\frac{1}{6}\pi$'])

    def plot(self, x, y, **kwargs):
        if self.use_radians:
            x = np.degrees(x)
        super(PolarPlot, self).plot(x, y, **kwargs)

    def histogram(self, counts, bin_edges, linestyle='solid'):
        """Plot a polar histogram.

        The user needs to supply the histogram.  This method only plots
        the results.  You can use NumPy's histogram function.

        :param counts: array containing the count values.
        :param bin_edges: array containing the bin edges.
        :param linestyle: the line style used to connect the data points.
            May be None, or any line style accepted by TikZ (e.g. solid,
            dashed, dotted, thick, or even combinations like
            "red,thick,dashed").

        Example::

            >>> plot = artist.PolarPlot()
            >>> x = np.random.uniform(0, 360, size=1000)
            >>> n, bins = np.histogram(x, bins=np.linspace(0, 360, 37))
            >>> plot.histogram(n, bins)

        """
        if len(bin_edges) - 1 != len(counts):
            raise RuntimeError(
                "The length of bin_edges should be length of counts + 1")

        x = []
        y = []

        if self.use_radians:
            circle = 2 * np.pi
        else:
            circle = 360.

        step = circle / 1800.

        for i in range(len(bin_edges) - 1):
            for bin_edge in np.arange(bin_edges[i], bin_edges[i + 1],
                                      step=step):
                x.append(bin_edge)
                y.append(counts[i])
            x.append(bin_edges[i + 1])
            y.append(counts[i])

        # If last edge is same as first bin edge, connect the ends.
        if bin_edges[-1] % circle == bin_edges[0] % circle:
            x.append(bin_edges[0])
            y.append(counts[0])

        self.plot(x, y, mark=None, linestyle=linestyle)

    def set_xlimits(self, min=None, max=None):
        """Do not allow setting x limits, it messes with the axes."""

        pass



import numpy as np

from artist import Plot, MultiPlot


def main():

    x = np.random.normal(0, 50, 50000)
    y = np.random.normal(0, 15, 50000)

    ranges = ([(-100, 0), (-50, 0)],
              [(0, 100), (-50, 0)],
              [(-100, 0), (0, 50)],
              [(0, 100), (0, 50)])
    types = ('reverse_bw', 'bw', 'reverse_bw', 'bw')
    bitmaps = (True, True, False, False)

    plot = Plot()
    for r, t, b in zip(ranges, types, bitmaps):
        n, xbins, ybins = np.histogram2d(x, y, bins=15, range=r)
        plot.histogram2d(n, xbins, ybins, type=t, bitmap=b)

    plot.save('histogram2d')

    plot = MultiPlot(2, 2, width=r'.4\linewidth')
    subplot_idxs = [(1, 0), (1, 1), (0, 0), (0, 1)]
    for idx, r, t, b in zip(subplot_idxs, ranges, types, bitmaps):
        p = plot.get_subplot_at(*idx)
        n, xbins, ybins = np.histogram2d(x, y, bins=15, range=r)
        p.histogram2d(n, xbins, ybins, type=t, bitmap=b)
    plot.show_yticklabels_for_all([(1, 0), (0, 1)])
    plot.show_xticklabels_for_all([(1, 0), (0, 1)])

    plot.save('multi_histogram2d')


if __name__ == '__main__':
    main()



"""Artist makes beautiful plots.

Artist is a 2D plotting library for Python.  It's main focus is the
output.  Artist creates a LaTeX file which can be included in your paper
or thesis.  The code needs TikZ and PGFPlots to compile and your plots
will tightly integrate with your main text.  Artist provides an
easy-to-use and clean Python interface and can compile your plot
on-the-fly as a stand-alone PDF file.


Rationale
---------

Artist enables you to visualize the results of your data analysis.  The
quality of your plots should reflect the quality of your analysis.  With
most software, this is hardly possible and the term 'publication quality'
takes on an entirely new meaning.  As a result, many papers and theses
suffer from inconsistent and generally poor-quality plots.

Fortunately, some solutions are available.  For LaTeX users, one can use
PGF/TikZ for generating figures and plots.  This ensures a very consistent
display throughout your document.  PGFPLOTS builds on that to provide a
user-friendly interface for many kinds of plots and to allow extensive
customization.

For many users, however, it is more convenient to use a programmatic
interface from your favorite programming language.  For Python, such an
interface is available in Artist.

Artist can be used in place of other plotting libraries, but the output is
a LaTeX file requiring PGF/TikZ and PGFPLOTS.  Previewing the output is
possible by means of a simple method which renders the plot as a PDF.

The style of the plots is based on the work of William S. Cleveland.


Contents
--------

The following modules are included:

:mod:`~artist.plot`
    contains the Plot and PolarPlot classes for creating plots.

:mod:`~artist.multi_plot`
    contains the MultiPlot class for creating plots comprised of multiple
    subplots.

:mod:`~artist.simple_smooth`
    contains a very basic smoothing algorithm.

:mod:`~artist.recursive_smooth`
    contains a more sophisticated recursive smoothing algorithm.

:mod:`~artist.utils`
    contains utility functions for e.g. automagic naming of output files
    based on the name of the function creating the plot.

"""
from .plot import Plot, PolarPlot
from .multi_plot import MultiPlot
from .recursive_smooth import smooth

# Backwards compatibility
from .plot import Plot as GraphArtist


__all__ = ['Plot', 'PolarPlot', 'MultiPlot', 'smooth', 'GraphArtist']



import numpy as np

from artist import PolarPlot


def main():
    plot = PolarPlot()

    # Full range histogram
    x = np.random.uniform(0, 360, 10000)
    n, bins = np.histogram(x, bins=np.linspace(0, 360, 181))
    plot.histogram(n, bins)

    # Offset range histogram
    x = np.random.normal(400, 130, 2500)
    n, bins = np.histogram(x, bins=np.linspace(270, 630, 91))
    plot.histogram(n, bins, linestyle='blue')

    # Part range histogram
    x = np.random.uniform(90, 300, 2000)
    n, bins = np.histogram(x, bins=np.linspace(90, 300, 22))
    plot.histogram(n, bins, linestyle='red')

    plot.save('polar_histogram')


if __name__ == '__main__':
    main()



import numpy as np
from numpy import sqrt
import pylab as plt

from artist import Plot

# For TeX rendering
plt.rcParams['font.serif'] = 'Computer Modern'
plt.rcParams['font.sans-serif'] = 'Computer Modern'
plt.rcParams['font.family'] = 'sans-serif'
plt.rcParams['figure.figsize'] = [4 * x for x in (1, 2. / 3)]
plt.rcParams['figure.subplot.left'] = 0.175
plt.rcParams['figure.subplot.bottom'] = 0.175
plt.rcParams['font.size'] = 10
plt.rcParams['legend.fontsize'] = 'small'
plt.rcParams['text.usetex'] = True


def main():
    e_data = np.genfromtxt('data/estar-plastic-scint.txt', skip_header=8,
                           usecols=(0, 1))
    e_kin_E = e_data[:, 0]
    e_mass = .511
    e_beta_gamma = sqrt(e_kin_E / e_mass * (e_kin_E / e_mass + 2))
    e_loss = e_data[:, 1]

    mu_data = np.genfromtxt('data/muonloss_216.dat', usecols=(1, 2))
    mu_p = mu_data[:, 0]
    mu_mass = 105.658367
    mu_beta_gamma = mu_p / mu_mass
    mu_loss = mu_data[:, 1]

    # Matplotlib
    plt.figure()
    plt.loglog(e_beta_gamma, e_loss, label="e")
    plt.loglog(mu_beta_gamma, mu_loss, label=r"$\mu$")
    plt.xlabel(r'$\beta\gamma$')
    plt.ylabel('Stopping Power [MeV cm$^2$ g$^{-1}$]')
    plt.legend()
    plt.savefig('stopping-power-mpl.pdf')

    # Artist
    plot = Plot(axis='loglog')
    plot.plot(e_beta_gamma, e_loss, mark=None)
    plot.plot(mu_beta_gamma, mu_loss, mark=None)
    plot.set_xlabel(r'$\beta\gamma$')
    plot.set_ylabel(r'Stopping Power $\left[\si{\mega\electronvolt'
                    r'\centi\meter\squared\per\gram}\right]$')

    plot.add_pin_at_xy(e_beta_gamma, e_loss, 'e', location='below right',
                       relative_position=.8)
    plot.add_pin_at_xy(mu_beta_gamma, mu_loss, r'$\mu$', 'above left')

    plot.set_xlimits(1e-2, 1e8)
    plot.set_ylimits(min=1)
    plot.set_logxticks(range(-2, 9, 2))
    plot.save('stopping-power')


if __name__ == '__main__':
    main()



import numpy as np

from artist import Plot


def main():
    x, t25, t50, t75 = np.loadtxt('data/DIR-boxplot_arrival_times-1.txt')

    graph = Plot()
    graph.plot(x, t50, mark='*')
    graph.shade_region(x, t25, t75)
    graph.set_xlabel(r"Core distance [\si{\meter}]")
    graph.set_ylabel(r"Arrival time delay [\si{\nano\second}]")
    graph.set_ylimits(min=0)
    graph.save('shower-front')


if __name__ == '__main__':
    main()



import numpy as np

from artist import Plot


def main():
    gamma = np.genfromtxt('data/showere15-proton.t2001', usecols=(1, 2))
    e = np.genfromtxt('data/showere15-proton.t2205', usecols=(1, 2))
    mu = np.genfromtxt('data/showere15-proton.t2207', usecols=(1, 2))

    graph = Plot(axis='loglog')

    graph.plot(gamma[:, 0], gamma[:, 1], mark=None)
    graph.add_pin(r'$\gamma$')

    graph.plot(e[:, 0], e[:, 1], mark=None)
    graph.add_pin('e')

    graph.plot(mu[:, 0], mu[:, 1], mark=None)
    graph.add_pin(r'$\mu$')

    graph.set_xlabel(r"Core distance [\si{\meter}]")
    graph.set_ylabel(r"Particle density [\si{\per\square\meter}]")
    graph.set_logyticks(range(-6, 3, 2))
    graph.save('eas-lateral')


if __name__ == '__main__':
    main()



from artist import Plot, MultiPlot


def main():
    """Event display for an event of station 503

    Date        Time      Timestamp   Nanoseconds
    2012-03-29  10:51:36  1333018296  870008589

    Number of MIPs
    35.0  51.9  35.8  78.9

    Arrival time
    15.0  17.5  20.0  27.5

    """
    # Detector positions in ENU relative to the station GPS
    x = [-6.34, -2.23, -3.6, 3.46]
    y = [6.34, 2.23, -3.6, 3.46]

    # Scale mips to fit the graph
    n = [35.0, 51.9, 35.8, 78.9]

    # Make times relative to first detection
    t = [15., 17.5, 20., 27.5]
    dt = [ti - min(t) for ti in t]

    plot = Plot()
    plot.scatter([0], [0], mark='triangle')
    plot.add_pin_at_xy(0, 0, 'Station 503', use_arrow=False, location='below')
    plot.scatter_table(x, y, dt, n)

    plot.set_scalebar(location="lower right")
    plot.set_colorbar('$\Delta$t [ns]')
    plot.set_axis_equal()
    plot.set_mlimits(max=16.)
    plot.set_slimits(min=10., max=100.)

    plot.set_xlabel('x [m]')
    plot.set_ylabel('y [m]')

    plot.save('event_display')


    # Add event by Station 508
    # Detector positions in ENU relative to the station GPS
    x508 = [6.12, 0.00, -3.54, 3.54]
    y508 = [-6.12, -13.23, -3.54, 3.54]

    # Event GPS timestamp: 1371498167.016412100
    # MIPS
    n508 = [5.6, 16.7, 36.6, 9.0]
    # Arrival Times
    t508 = [15., 22.5, 22.5, 30.]
    dt508 = [ti - min(t508) for ti in t508]

    plot = MultiPlot(1, 2, width=r'.33\linewidth')
    plot.set_xlimits_for_all(min=-10, max=15)
    plot.set_ylimits_for_all(min=-15, max=10)
    plot.set_mlimits_for_all(min=0., max=16.)
    plot.set_colorbar('$\Delta$t [ns]', False)
    plot.set_colormap('blackwhite')
    plot.set_scalebar_for_all(location="upper right")

    p = plot.get_subplot_at(0, 0)
    p.scatter([0], [0], mark='triangle')
    p.add_pin_at_xy(0, 0, 'Station 503', use_arrow=False, location='below')
    p.scatter_table(x, y, dt, n)
    p.set_axis_equal()

    p = plot.get_subplot_at(0, 1)
    p.scatter([0], [0], mark='triangle')
    p.add_pin_at_xy(0, 0, 'Station 508', use_arrow=False, location='below')
    p.scatter_table(x508, y508, dt508, n508)
    p.set_axis_equal()

    plot.show_yticklabels_for_all([(0, 0)])
    plot.show_xticklabels_for_all([(0, 0), (0, 1)])

    plot.set_xlabel('x [m]')
    plot.set_ylabel('y [m]')

    plot.save('multi_event_display')


if __name__ == '__main__':
    main()



"""Recursive linear interpolation smoother."""

import numpy as np


def smooth(x, y, degree=1, logx=False, logy=False):
    """Smooth y-values and return new x, y pair.

    :param x, y: data values
    :param degree: degree of smoothing

    Smooth data by using a recursive linear interpolation technique.  For
    degree = 0, return the original values.  For degree = 1, generate a
    evenly spaced sequence of x-values, with the length equal to the
    original length.  The y-values are linearly interpolated for these
    x-values.  For degree >= 2, calls itself with degree - 1, then
    calculates new x-values by taking the averages of the returned
    x-values, and calculates new y-values by linear interpolation.  The
    return values are thus reduced in length by one sample.

    """
    if degree == 0:
        return x, y
    else:
        if logx:
            x = np.log10(x)
        if logy:
            y = np.log10(y)

        if degree == 1:
            # generate new linearly spaced x-points
            smoothed_x = np.linspace(min(x), max(x), len(x))
            # generate new y-points using linear interpolation
            smoothed_y = np.interp(smoothed_x, x, y)
        else:
            # smooth data by linear interpolation
            x, y = smooth(x, y, degree - 1)
            smoothed_x = (x[:-1] + x[1:]) / 2
            smoothed_y = np.interp(smoothed_x, x, y)

        if logx:
            smoothed_x = 10 ** smoothed_x
        if logy:
            smoothed_y = 10 ** smoothed_y

        return smoothed_x, smoothed_y



"""Utility functions."""

import inspect
import os

import numpy as np


# global suffix and prefix for all subsequent graph names
__prefix = ''
__suffix = ''


def set_prefix(prefix):
    """Set global prefix for all graph names.

    :param prefix: prefix for all plot names
    :type prefix: string

    """
    global __prefix
    __prefix = prefix


def set_suffix(suffix):
    """Set global suffix for all graph names.

    :param prefix: suffix for all plot names
    :type prefix: string

    """

    global __suffix
    __suffix = suffix


def get_callers_name(level=2):
    """Return name of caller.

    Return the name of the caller, optionally nested multiple levels deep.

    :param level: level of recursion.  level=0 is this method, level=1 is
        the caller, and level=2 is the caller's parent.

    """
    frame_record = inspect.stack()[level]
    function_name = frame_record[3]
    return function_name


def create_graph_name(suffix='', dirname=None):
    """Create a graph name using the name of the caller.

    :param suffix: optional suffix to add to name
    :param dirname: optional directory name

    :return: path for the named graph
    :rtype: string

    """
    if suffix:
        suffix = '-%s' % suffix
    caller = get_callers_name(level=3)
    name = '%s%s%s%s' % (__prefix, caller, suffix, __suffix)
    if dirname:
        name = os.path.join(dirname, name)
    return name


def save_graph(graph, suffix='', dirname=None, pdf=False):
    """Save a graph using caller's name.

    :type graph: GraphArtist instance
    :param suffix: optional suffix to add to name
    :param dirname: optional directory name
    :param pdf: if True, the saved graph is additionally rendered and
        saved as a pdf, alongside the LaTeX file.

    """
    name = create_graph_name(suffix, dirname)
    graph.save(name)
    if pdf:
        graph.save_as_pdf(name)


def save_data(data, suffix='', dirname=None):
    """Save a dataset using caller's name.

    :param data: a list or numpy array containing the data
    :param suffix: optional suffix to add to name
    :param dirname: optional directory name

    """
    if type(data) == list:
        data = np.array(data).T

    name = create_graph_name(suffix, dirname) + '.txt'
    np.savetxt(name, data)



"""Create a plot containing multiple subplots.

Contents
--------

:class:`MultiPlot`
    Create a plot containing multiple subplots.

:class:`SubPlotContainer`
    Adds metadata to a SubPlot instance for use in a MultiPlot.

"""

from math import sqrt
import warnings

import jinja2

from .plot import BasePlotContainer, SubPlot


class MultiPlot(BasePlotContainer):

    """Create a plot containing multiple subplots.

    This class creates a 2D plot containing multiple subplots.  The number
    of rows and columns can be specified and some of the subplot
    rectangles can remain empty (nothing is drawn, not even an axis
    rectangle).  Its various methods add data, annotations and options
    which is stored in class variables.  Finally, the plot can be rendered
    using the Jinja2 templating engine resulting in a LaTeX or PDF file.

    """

    def __init__(self, rows, columns, axis='',
                 width=r'.67\linewidth', height=None):
        environment = jinja2.Environment(loader=jinja2.PackageLoader(
            'artist', 'templates'), finalize=self._convert_none)
        self.template = environment.get_template('multi_plot.tex')
        self.document_template = environment.get_template(
            'document.tex')

        self.rows = rows
        self.columns = columns
        self.xmode, self.ymode = self._get_axis_modes(axis)
        self.width = width
        self.height = height
        self.xlabel = None
        self.ylabel = None
        self.limits = {'xmin': None, 'xmax': None,
                       'ymin': None, 'ymax': None,
                       'mmin': None, 'mmax': None,
                       'smin': None, 'smax': None}
        self.ticks = {'x': [], 'y': []}
        self.colorbar = None
        self.colormap = None
        self.external_filename = None

        self.subplots = []
        for i in range(rows):
            for j in range(columns):
                self.subplots.append(SubPlotContainer(i, j, self.xmode,
                                                      self.ymode))

    def save_assets(self, dest_path):
        """Save plot assets alongside dest_path.

        Some plots may have assets, like bitmap files, which need to be
        saved alongside the rendered plot file.

        :param dest_path: path of the main output file.

        """
        for idx, subplot in enumerate(self.subplots):
            subplot.save_assets(dest_path, suffix='_%d' % idx)

    def set_empty(self, row, column):
        """Keep one of the subplots completely empty.

        :param row,column: specify the subplot.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_empty()

    def set_empty_for_all(self, row_column_list):
        """Keep all specified subplots completely empty.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None

        """
        for row, column in row_column_list:
            self.set_empty(row, column)

    def set_title(self, row, column, text):
        """Set a title text.

        :param row,column: specify the subplot.
        :param text: title text.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_title(text)

    def set_label(self, row, column, text, location='upper right',
                  style=None):
        """Set a label for the subplot.

        :param row,column: specify the subplot.
        :param text: the label text.
        :param location: the location of the label inside the plot.  May
            be one of 'center', 'upper right', 'lower right', 'upper
            left', 'lower left'.
        :param style: any TikZ style to style the text.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_label(text, location, style)

    def show_xticklabels(self, row, column):
        """Show the x-axis tick labels for a subplot.

        :param row,column: specify the subplot.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.show_xticklabels()

    def show_xticklabels_for_all(self, row_column_list=None):
        """Show the x-axis tick labels for all specified subplots.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None

        """
        if row_column_list is None:
            for subplot in self.subplots:
                subplot.show_xticklabels()
        else:
            for row, column in row_column_list:
                self.show_xticklabels(row, column)

    def show_yticklabels(self, row, column):
        """Show the y-axis tick labels for a subplot.

        :param row,column: specify the subplot.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.show_yticklabels()

    def show_yticklabels_for_all(self, row_column_list=None):
        """Show the y-axis tick labels for all specified subplots.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None

        """
        if row_column_list is None:
            for subplot in self.subplots:
                subplot.show_yticklabels()
        else:
            for row, column in row_column_list:
                self.show_yticklabels(row, column)

    def set_xticklabels_position(self, row, column, position):
        """Specify the position of the axis tick labels.

        This is generally only useful for multiplots containing only one
        row.  This can be used to e.g. alternatively draw the tick labels
        on the bottom or the top of the subplot.

        :param row,column: specify the subplot.
        :param position: 'top' or 'bottom' to specify the position of the
            tick labels.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_xticklabels_position(position)

    def set_yticklabels_position(self, row, column, position):
        """Specify the position of the axis tick labels.

        This is generally only useful for multiplots containing only one
        column.  This can be used to e.g. alternatively draw the tick
        labels on the left or the right of the subplot.

        :param row,column: specify the subplot.
        :param position: 'left' or 'right' to specify the position of the
            tick labels.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_yticklabels_position(position)

    def set_xlimits(self, row, column, min=None, max=None):
        """Set x-axis limits of a subplot.

        :param row,column: specify the subplot.
        :param min: minimal axis value
        :param max: maximum axis value

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_xlimits(min, max)

    def set_xlimits_for_all(self, row_column_list=None, min=None, max=None):
        """Set x-axis limits of specified subplots.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param min: minimal axis value
        :param max: maximum axis value

        """
        if row_column_list is None:
            self.limits['xmin'] = min
            self.limits['xmax'] = max
        else:
            for row, column in row_column_list:
                self.set_xlimits(row, column, min, max)

    def set_ylimits(self, row, column, min=None, max=None):
        """Set y-axis limits of a subplot.

        :param row,column: specify the subplot.
        :param min: minimal axis value
        :param max: maximum axis value

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_ylimits(min, max)

    def set_ylimits_for_all(self, row_column_list=None, min=None, max=None):
        """Set y-axis limits of specified subplots.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param min: minimal axis value
        :param max: maximum axis value

        """
        if row_column_list is None:
            self.limits['ymin'] = min
            self.limits['ymax'] = max
        else:
            for row, column in row_column_list:
                self.set_ylimits(row, column, min, max)

    def set_mlimits(self, row, column, min=None, max=None):
        """Set limits for the point meta (colormap).

        Point meta values outside this range will be clipped.

        :param min: value for start of the colormap.
        :param max: value for end of the colormap.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_mlimits(min, max)

    def set_mlimits_for_all(self, row_column_list=None, min=None, max=None):
        """Set limits for point meta (colormap) for specified subplots.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param min: value for start of the colormap.
        :param max: value for end of the colormap.

        """
        if row_column_list is None:
            self.limits['mmin'] = min
            self.limits['mmax'] = max
        else:
            for row, column in row_column_list:
                self.set_mlimits(row, column, min, max)

    def set_slimits(self, row, column, min, max):
        """Set limits for the point sizes.

        :param min: point size for the lowest value.
        :param max: point size for the highest value.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_slimits(min, max)

    def set_slimits_for_all(self, row_column_list=None, min=None, max=None):
        """Set point size limits of specified subplots.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param min: point size for the lowest value.
        :param max: point size for the highest value.

        """
        if min is None or max is None:
            raise Exception('Both min and max are required.')
        if row_column_list is None:
            self.limits['smin'] = sqrt(min)
            self.limits['smax'] = sqrt(max)
        else:
            for row, column in row_column_list:
                self.set_slimits(row, column, min, max)

    def set_xticks(self, row, column, ticks):
        """Manually specify the x-axis tick values.

        :param row,column: specify the subplot.
        :param ticks: list of tick values.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_xticks(ticks)

    def set_xticks_for_all(self, row_column_list=None, ticks=None):
        """Manually specify the x-axis tick values.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param ticks: list of tick values.

        """
        if row_column_list is None:
            self.ticks['x'] = ticks
        else:
            for row, column in row_column_list:
                self.set_xticks(row, column, ticks)

    def set_logxticks(self, row, column, logticks):
        """Manually specify the x-axis log tick values.

        :param row,column: specify the subplot.
        :param logticks: logarithm of the locations for the ticks along the
            axis.

        For example, if you specify [1, 2, 3], ticks will be placed at 10,
        100 and 1000.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_logxticks(logticks)

    def set_logxticks_for_all(self, row_column_list=None, logticks=None):
        """Manually specify the x-axis log tick values.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param logticks: logarithm of the locations for the ticks along the
            axis.

        For example, if you specify [1, 2, 3], ticks will be placed at 10,
        100 and 1000.

        """
        if row_column_list is None:
            self.ticks['x'] = ['1e%d' % u for u in logticks]
        else:
            for row, column in row_column_list:
                self.set_logxticks(row, column, logticks)

    def set_yticks(self, row, column, ticks):
        """Manually specify the y-axis tick values.

        :param row,column: specify the subplot.
        :param ticks: list of tick values.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_yticks(ticks)

    def set_yticks_for_all(self, row_column_list=None, ticks=None):
        """Manually specify the y-axis tick values.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param ticks: list of tick values.

        """
        if row_column_list is None:
            self.ticks['y'] = ticks
        else:
            for row, column in row_column_list:
                self.set_yticks(row, column, ticks)

    def set_logyticks(self, row, column, logticks):
        """Manually specify the y-axis log tick values.

        :param row,column: specify the subplot.
        :param logticks: logarithm of the locations for the ticks along the
            axis.

        For example, if you specify [1, 2, 3], ticks will be placed at 10,
        100 and 1000.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_logyticks(logticks)

    def set_logyticks_for_all(self, row_column_list=None, logticks=None):
        """Manually specify the y-axis log tick values.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :type row_column_list: list or None
        :param logticks: logarithm of the locations for the ticks along the
            axis.

        For example, if you specify [1, 2, 3], ticks will be placed at 10,
        100 and 1000.

        """
        if row_column_list is None:
            self.ticks['y'] = ['1e%d' % u for u in logticks]
        else:
            for row, column in row_column_list:
                self.set_logyticks(row, column, logticks)

    def get_subplot_at(self, row, column):
        """Return the subplot at row, column position.

        :param row,column: specify the subplot.

        """
        idx = row * self.columns + column
        return self.subplots[idx]

    def render(self, template=None):
        """Render the plot using a template.

        Once the plot is complete, it needs to be rendered.  Artist uses
        the Jinja2 templating engine.  The default template results in a
        LaTeX file which can be included in your document.

        :param template: a user-supplied template or None.
        :type template: string or None.
        :returns: the rendered template as string.

        """
        if not template:
            template = self.template

        response = template.render(rows=self.rows, columns=self.columns,
                                   xmode=self.xmode, ymode=self.ymode,
                                   width=self.width, height=self.height,
                                   xlabel=self.xlabel, ylabel=self.ylabel,
                                   limits=self.limits, ticks=self.ticks,
                                   colorbar=self.colorbar,
                                   colormap=self.colormap,
                                   external_filename=self.external_filename,
                                   subplots=self.subplots,
                                   plot_template=self.template)
        return response

    def set_xlabel(self, text):
        """Set a label for the x-axis.

        :param text: text of the label.

        """
        self.xlabel = text

    def set_ylabel(self, text):
        """Set a label for the y-axis.

        :param text: text of the label.

        """
        self.ylabel = text

    def set_subplot_xlabel(self, row, column, text):
        """Set a label for the x-axis of a subplot.

        :param row,column: specify the subplot.
        :param text: text of the label.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_xlabel(text)

    def set_subplot_ylabel(self, row, column, text):
        """Set a label for the y-axis of a subplot.

        :param row,column: specify the subplot.
        :param text: text of the label.

        """
        subplot = self.get_subplot_at(row, column)
        subplot.set_ylabel(text)

    def set_scalebar_for_all(self, row_column_list=None,
                             location='lower right'):
        """Show marker area scale for subplots.

        :param row_column_list: a list containing (row, column) tuples to
            specify the subplots, or None to indicate *all* subplots.
        :param location: the location of the label inside the plot.  May
            be one of 'center', 'upper right', 'lower right', 'upper
            left', 'lower left'.

        """
        if row_column_list is None:
            for subplot in self.subplots:
                subplot.set_scalebar(location)
        else:
            for row, column in row_column_list:
                subplot = self.get_subplot_at(row, column)
                subplot.set_scalebar(location)

    def set_colorbar(self, label='', horizontal=False):
        """Show the colorbar, it will be attached to the last plot.

        Not for the histogram2d, only for the scatter_table.
        Global mlimits should be set for this to properly reflect the
        colormap of each subplot.

        :param label: axis label for the colorbar.
        :param horizontal: boolean, if True the colobar will be horizontal.

        """
        if self.limits['mmin'] is None or self.limits['mmax'] is None:
            warnings.warn('Set (only) global point meta limits to ensure the '
                          'colorbar is correct for all subplots.')
        self.colorbar = {'label': label,
                         'horizontal': horizontal}

    def set_colormap(self, name):
        """Choose a colormap for all subplots.

        :param name: name of the colormap to use. (e.g. hot, cool, blackwhite,
                     greenyellow). If None a coolwarm colormap is used.

        """
        self.colormap = name


class SubPlotContainer(SubPlot):

    """Add metadata to a SubPlot, for inclusion in MultiPlots.

    This class adds some metadata to a subplot, including the mode of the
    plot and whether to show tick labels.  For use in MultiPlots, the row
    and column position are attributes.

    """

    def __init__(self, row, column, xmode, ymode):
        self.row = row
        self.column = column
        self.xmode, self.ymode = xmode, ymode
        self.empty = False
        self.show_xticklabel = False
        self.show_yticklabel = False
        self.xticklabel_pos = None
        self.yticklabel_pos = None
        super(SubPlotContainer, self).__init__()

    def set_empty(self):
        """Set the plot to be completely empty."""

        self.empty = True

    def show_xticklabels(self):
        """Show the x-axis tick labels for this subplot."""

        self.show_xticklabel = True

    def show_yticklabels(self):
        """Show the y-axis tick labels for this subplot."""

        self.show_yticklabel = True

    def set_xticklabels_position(self, position):
        """Specify the position of the axis tick labels.

        This is generally only useful for multiplots containing only one
        row.  This can be used to e.g. alternatively draw the tick labels
        on the bottom or the top of the subplot.

        :param position: 'top' or 'bottom' to specify the position of the
            tick labels.

        """
        pgfplots_translation = {'top': 'right', 'bottom': 'left'}
        fixed_position = pgfplots_translation[position]

        self.xticklabel_pos = fixed_position

    def set_yticklabels_position(self, position):
        """Specify the position of the axis tick labels.

        This is generally only useful for multiplots containing only one
        column.  This can be used to e.g. alternatively draw the tick
        labels on the left or the right of the subplot.

        :param position: 'left' or 'right' to specify the position of the
            tick labels.

        """
        self.yticklabel_pos = position



import numpy as np
from numpy import pi, sin, cos, tan

import artist


def main():
    np.random.seed(1)

    graph = artist.MultiPlot(2, 3, width=r'.25\linewidth',
                             height=r'.25\linewidth')
    x = np.linspace(-pi, pi)

    subplot = graph.get_subplot_at(0, 1)
    subplot.plot(x, sin(x), mark=None)
    subplot.add_pin('$\sin(x)$', relative_position=.5)

    subplot = graph.get_subplot_at(1, 0)
    subplot.plot(x, cos(x), mark=None)
    subplot.add_pin_at_xy(1, .5, '$\cos(x)$', use_arrow=True)

    subplot = graph.get_subplot_at(1, 2)
    subplot.plot(x, tan(x), mark=None)

    x = np.random.normal(size=1000)
    n, bins = np.histogram(x, bins=20)
    subplot = graph.get_subplot_at(0, 0)
    subplot.histogram(n, bins)
    subplot.add_pin('histogram', location='left', relative_position=.5)

    x = range(5)
    lower = np.random.uniform(-2, -1, size=5)
    median = np.random.uniform(-.5, .5, size=5)
    upper = np.random.uniform(1, 2, size=5)
    subplot = graph.get_subplot_at(0, 2)
    subplot.plot(x, median, mark='*')
    subplot.shade_region(x, lower, upper)

    subplot = graph.get_subplot_at(1, 1)
    subplot.plot(range(5), np.random.normal(size=5))

    graph.show_xticklabels_for_all([(0, 0), (1, 1), (0, 2)])
    graph.show_yticklabels(0, 0)
    graph.show_yticklabels(1, 2)

    graph.set_ylabel(r"Particle density [\si{\per\square\meter}]")
    graph.set_xlabel(r"Core distance [\si{\meter}]")

    graph.set_title(0, 1, "Nice plot")

    graph.set_subplot_xlabel(0, 0, "number")
    graph.set_subplot_ylabel(1, 2, r"$\tan x$")

    graph.save('multiplot')


if __name__ == '__main__':
    main()



import numpy as np

from artist import PolarPlot


def main():

    directions = np.genfromtxt('data/discrete-directions.txt',
                               names=['azimuth', 'zenith'])

    plot = PolarPlot(use_radians=True)
    plot.scatter(directions['azimuth'], directions['zenith'],
                 markstyle='mark size=.75pt')

    plot.set_xlabel('Azimuth [rad]')
    plot.set_ylabel('Zenith [rad]')

    plot.save('discrete_directions')


if __name__ == '__main__':
    main()



"""Simple averaging smoother."""

import numpy as np


def smooth(x, y, degree=3):
    """Smooth y-values and return new x, y pair."""

    if degree == 1:
        return x, y
    elif not isodd(degree):
        raise ValueError("Degree must be an odd number")
    else:
        window_width = int(degree / 2)
        data_size = len(y) - degree + 1

        smoothed_y = np.zeros(data_size)
        for idx in range(degree):
            smoothed_y += y[idx:idx + data_size]
        smoothed_y /= degree

        smoothed_x = x[window_width:-window_width]

        return smoothed_x, smoothed_y


def isodd(x):
    """Determine if number is odd.

    :return: boolean

    """
    if x % 2 == 1:
        return True
    else:
        return False



import numpy as np
from PIL import Image

from artist import Plot


def main():
    stations = np.genfromtxt('data/cluster-utrecht-stations.txt',
                             names=['x', 'y'])
    image = Image.open('data/cluster-utrecht-background.png')

    graph = Plot(width=r'.75\linewidth', height=r'.5\linewidth')

    graph.scatter(stations['x'], stations['y'])
    graph.draw_image(image)

    graph.set_axis_equal()

    nw = ['%.4f' % i for i in (52.10650519075632, 5.053710938)]
    se = ['%.4f' % i for i in (52.05249047600099, 5.185546875)]

    graph.set_xlabel('Longitude [$^\circ$]')
    graph.set_xticks([0, image.size[0]])
    graph.set_xtick_labels([nw[1], se[1]])

    graph.set_ylabel('Latitude [$^\circ$]')
    graph.set_yticks([0, image.size[1]])
    graph.set_ytick_labels([se[0], nw[0]])

    graph.save('utrecht')


if __name__ == '__main__':
    main()



import numpy as np

from artist import Plot


def main():
    locations = np.genfromtxt(
        'data/SP-DIR-plot_sciencepark_cluster-detectors.txt',
        names=['x', 'y'])
    stations = np.genfromtxt(
        'data/SP-DIR-plot_sciencepark_cluster-stations.txt',
        names=['id', 'x', 'y'])

    graph = Plot()

    graph.scatter(locations['x'], locations['y'])
    graph.set_axis_equal()

    locations = ['right'] * len(stations)
    locations[0] = 'left'
    locations[5] = 'above right'
    locations = iter(locations)
    for num, x, y in stations:
        graph.add_pin_at_xy(x, y, int(num), location=next(locations),
                            use_arrow=False, style='gray,label distance=1ex')

    x = [stations['x'][u] for u in [0, 2, 5]]
    y = [stations['y'][u] for u in [0, 2, 5]]
    x.append(x[0])
    y.append(y[0])
    graph.plot(x, y, mark=None, linestyle='dashed')

    graph.add_pin_at_xy([x[0], x[1]], [y[0], y[1]], r'\SI{128}{\meter}',
                        relative_position=.4, location='below right',
                        use_arrow=False)
    graph.add_pin_at_xy([x[0], x[2]], [y[0], y[2]], r'\SI{151}{\meter}',
                        relative_position=.5, location='left',
                        use_arrow=False)
    graph.add_pin_at_xy([x[2], x[1]], [y[2], y[1]], r'\SI{122}{\meter}',
                        relative_position=.5, location='above right',
                        use_arrow=False)

    graph.set_xlabel(r"Distance [\si{\meter}]")
    graph.set_ylabel(r"Distance [\si{\meter}]")

    graph.save('sciencepark')


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(name = "artist",
      version = "0.15.0",
      packages = find_packages(),
      url = "http://github.com/davidfokkema/artist/",
      bugtrack_url='http://github.com/davidfokkema/artist/issues',
      license='GPLv3',
      author = "David Fokkema",
      author_email = "davidfokkema@icloud.com",
      description = "A plotting library for Python with LaTeX output",
      long_description=open('README.rst').read(),
      keywords=['plots', 'plotting', 'data visualization'],
      classifiers=['Intended Audience :: Science/Research',
                   'Intended Audience :: Education',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.4',
                   'Topic :: Scientific/Engineering :: Visualization',
                   'Topic :: Education',
                   'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'],
      install_requires = ['jinja2', 'numpy', 'Pillow'],
      package_data={'artist': ['templates/*.tex']},
)



import numpy as np

from artist import Plot


def main():
    leap = np.genfromtxt('data/leap-prot.dat', delimiter=',',
                         usecols=(0, 1), names=['E', 'F'])
    leap['E'] *= 1e6
    leap['F'] *= 1e3 * 2

    proton = read_data('data/proton', 1, 0)
    akeno_new_lo = read_data('data/akeno-new-lo', 0, 1)
    flys_eye = read_data('data/fe-new', 0, 1)
    yakutsk = read_data('data/yakustk', 1, 0)
    haverah = read_data('data/haverah', 1, 0)

    graph = Plot(axis='loglog', width=r'.5\linewidth', height=r'.65\linewidth')

    graph.plot(leap['E'], leap['F'], mark=None)
    graph.add_pin('LEAP', 'above right', use_arrow=True,
                  relative_position=.5)
    graph.plot(proton['E'], proton['F'], mark=None)
    graph.add_pin('PROTON', 'above right', use_arrow=True,
                  relative_position=.5)
    graph.plot(akeno_new_lo['E'], akeno_new_lo['F'], mark=None)
    graph.add_pin('AGASA', 'above right', use_arrow=True,
                  relative_position=.5)

    graph.plot(yakutsk['E'], yakutsk['F'], mark=None)
    graph.add_pin('Yakutsk', 'below left', use_arrow=True,
                  relative_position=.55)
    graph.plot(haverah['E'], haverah['F'], mark=None)
    graph.add_pin('Haverah Park', 'below left', use_arrow=True,
                  relative_position=.85)
    graph.plot(flys_eye['E'], flys_eye['F'], mark=None)
    graph.add_pin("Fly's Eye", 'below left', use_arrow=True,
                  relative_position=1.0)

    graph.set_xlabel(r"Energy [\si{\electronvolt}]")
    graph.set_ylabel(r"Flux [\si{\per\square\meter\per\steradian"
                     "\per\second\per\giga\electronvolt}]")

    x = np.logspace(11, 17)
    graph.plot(x, 1.5e29 * x ** -2.75, mark=None, linestyle='dashed')

    graph.set_logxticks(range(6, 22, 3))
    graph.save('spectrum')


def read_data(path, energy_col, flux_col):
    data = np.genfromtxt(path, usecols=(energy_col, flux_col),
                         names=['E', 'F'])
    data.sort()
    data['F'] /= (data['E'] / 1e9) ** 2.75
    return data


if __name__ == '__main__':
    main()
